#include <IRremote.h>  // IR리모콘을 사용하기 위한 헤더파일 호출
#define EA 3  // 모터드라이버 EA 핀, 아두이노 디지털 3번 핀에 연결
#define EB 11  // 모터드라이버 EB 핀, 아두이노 디지털 11번 핀에 연결
#define M_IN1 4  // 모터드라이버 IN1 핀, 아두이노 디지털 4번 핀에 연결
#define M_IN2 5  // 모터드라이버 IN2 핀, 아두이노 디지털 5번 핀에 연결
#define M_IN3 13  // 모터드라이버 IN3 핀, 아두이노 디지털 13번 핀에 연결
#define M_IN4 12  // 모터드라이버 IN4 핀, 아두이노 디지털 12번 핀에 연결
int motorA_vector = 1;  // 모터의 회전방향이 반대일 시 0을 1로, 1을 0으로 바꿔주면 모터의 회전방향이 바뀜.
int motorB_vector = 1;  // 모터의 회전방향이 반대일 시 0을 1로, 1을 0으로 바꿔주면 모터의 회전방향이 바뀜.
int motor_speed = 255;  // 모터 스피드 0 ~ 255

int RECV_PIN = 14;  // 적외선 수신센서 모듈과 A0 연결
IRrecv irrecv(RECV_PIN);  // 적외선 수신 핀 지정
decode_results results;
void setup()  // 초기화
{
  pinMode(EA, OUTPUT);  // EA 핀 출력 설정
  pinMode(EB, OUTPUT);  // EB 핀 출력 설정
  pinMode(M_IN1, OUTPUT);  // IN1 핀 출력 설정
  pinMode(M_IN2, OUTPUT);  // IN2 핀 출력 설정
  pinMode(M_IN3, OUTPUT);  // IN3 핀 출력 설정
  pinMode(M_IN4, OUTPUT);  // IN4 핀 출력 설정
  irrecv.enableIRIn();  // 적외선 수신센서 활성화
}
void loop()
{
  if (irrecv.decode(&results)) {
    switch (results.value) {  // 수신 받은 적외선 값과 비교
      case 0x00FF18E7:  // Key 2 - 전진
        motorA_con( motor_speed );  // 모터A 정방향
        motorB_con( motor_speed );  // 모터B 정방향
        break;
      case 0x00FF10EF:  // Key 4 - 좌회전
        motorA_con( motor_speed );  // 모터A 정방향
        motorB_con( 0 );  // 모터B 정지
        break;
      case 0x00FF38C7:  // Key 5 - 정지
        motorA_con( 0 );  // 모터A 정지
        motorB_con( 0 );  // 모터B 정지
        break;
      case 0x00FF5AA5:  // Key 6 - 우회전
        motorA_con( 0 );  // 모터A 정지
        motorB_con( motor_speed );  // 모터B 정회전
        break;
      case 0x00FF4AB5:  // Key 8 - 후진
        motorA_con( -motor_speed );  // 모터A 역회전
        motorB_con( -motor_speed );  // 모터B 역회전
        break;
      default:
        break;
    }
    irrecv.resume();
  }
}
void motorA_con(int M1)  // 모터A 컨트롤 함수
{
  if (M1 > 0)  // M1이 양수일 때, 모터A 정회전
  {
    digitalWrite(M_IN1, motorA_vector);  // IN1번에 HIGH(motorA_vector가 0이면 LOW)
    digitalWrite(M_IN2, !motorA_vector);  // IN2번에 LOW(motorA_vector가 0이면 HIGH)
  }
  else if (M1 < 0)  // M1이 음수일 대, 모터A 역회전
  {
    digitalWrite(M_IN1, !motorA_vector);  // IN1번에 LOW(motorA_vector가 0이면 HIGH)
    digitalWrite(M_IN2, motorA_vector);  // IN2번에 HIGH(motorA_vector가 0이면 LOW)
  }
  else  // 모터A 정지
  {
    digitalWrite(M_IN1, LOW);  // IN1번에 LOW
    digitalWrite(M_IN2, LOW);  // IN2번에 LOW
  }
  analogWrite(EA, abs(M1));  // M1의 절대값으로 모터A 속도 제어
}
void motorB_con(int M2)  // 모터B 컨트롤 함수
{
  if (M2 > 0)  // M2가 양수일 때, 모터B 정회전
  {
    digitalWrite(M_IN3, motorB_vector);  // IN3번에 HIGH(motorB_vector가 0이면 LOW)
    digitalWrite(M_IN4, !motorB_vector);  // IN4번에 LOW(motorB_vector가 0이면 HIGH)
  }
  else if (M2 < 0)  // M2가 음수일 때, 모터B 역회전
  {
    digitalWrite(M_IN3, !motorB_vector);  // IN3번에 LOW(motorB_vector가 0이면 HIGH)
    digitalWrite(M_IN4, motorB_vector);  // IN4번에 HIGH(motorB_vector가 0이면 LOW)
  }
  else  // 모터B 정지
  {
    digitalWrite(M_IN3, LOW);  // IN3번에 LOW
    digitalWrite(M_IN4, LOW);  // IN4번에 LOW
  }
  analogWrite(EB, abs(M2));  // M2의 절대값으로 모터B 속도 제어
}

